from turtle import *


def tree(trunkLen, levels, angle, shrinkFactor):
    '''Draws recursively a tree design with the following parameters:
    1. levels is the number of branches on any path from the root to a leaf
    2. trunkLen is the length of the base trunk of the tree
    3. angle is the angle from the trunk for each subtree
    4. shrinkFactor is the shrinking factor for each subtree.
    '''
    if levels == 0:
        pass 
    else:
        # Draw the trunk.
        fd(trunkLen)
        # Turn and draw the right subtree.
        rt(angle)
        newLen = trunkLen*shrinkFactor
        tree(newLen, levels-1, angle, shrinkFactor)
        # Turn and draw the left subtree.
        lt(angle * 2)
        tree(newLen, levels-1, angle, shrinkFactor)
        # Turn back and back up to root without drawing.
        rt(angle)
        pu()
        bk(trunkLen)
        pd()


def initializeTurtle(trunkLen):
    """initialize turtle on canvas before drawing"""
    setup(1000, 1000) # Create a turtle window 
    reset() # Clear any existing turtle drawings 
            # and reset turtle position & heading. 
    pensize(4) # Choose a pen thickness  
    speed(0) # Set the speed; 0=fastest, 1=slowest, 6=normal  

    #shape("turtle") # Make turtle shape a turtle (as opposed to arrow)
    #Turtle, by default, starts roughly in center of canvas
    padding = 100
    pu()
    setx(0)
    sety(-(trunkLen+padding))
    setheading(90)
    pd()
    
    # Magical statement to make turtle window come to top of screen.                              
    # getscreen()._root.attributes('-topmost', True)


def testTree(trunkLen, levels, angle, shrinkFactor):
    initializeTurtle(trunkLen)
    tree(trunkLen, levels, angle, shrinkFactor)
    getscreen().getcanvas().postscript(file="figs/tree({},{},{},{}).eps".format
                            (trunkLen, levels, angle, shrinkFactor))

if __name__ == '__main__':
    testTree(200, 3, 45, 0.6)
    #testTree(200, 7, 30, 0.8)
    #testTree(200, 7, 15, 0.8)
    #testTree(200, 10, 45, 0.7)
    #testTree(200, 10, 90, 0.68)
    exitonclick()
